using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Media.Media3D;
using System.Windows.Threading;
using FluidKit.Helpers;
using FluidKit.Helpers.Effects;


namespace WaterEffect
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class Window1 : System.Windows.Window
    {
        private int _meshDivisions = 100;
        private WaterRippleEffect _effect;

        public int MeshDivisions
        {
            get { return _meshDivisions; }
            set { _meshDivisions = value; }
        }

        public Window1()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Window1_Loaded);
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            double aspect = _viewport.ActualWidth / _viewport.ActualHeight;
            MeshGeometry3D mesh = MeshCreator.CreateMesh(MeshDivisions, MeshDivisions);
            mesh.Positions = new RectangularMeshFiller().FillMesh(MeshDivisions, MeshDivisions, aspect);
            _particleModel.Geometry = mesh;
            (_viewport.Camera as PerspectiveCamera).Position = new Point3D(0, 0, aspect * 0.86655);

            // Initialize
            _effect =  new WaterRippleEffect(MeshDivisions, MeshDivisions, mesh);
            _effect.EffectCompleted += new EffectCompletedHandler(ReplayEffect);
            _effect.PlayEffect();
        }

        void ReplayEffect()
        {
            _effect.PlayEffect();
        }

        private void DampingValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (_effect == null) return;
            _effect.Damping = e.NewValue;
        }


        private void DelayChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (_effect == null) return;
            _effect.UpdateInterval = e.NewValue;
        }

        private void WaterDepthChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (_effect == null) return;
            _effect.WaterDepth = e.NewValue;
        }


        private void RefractionIndexChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (_effect == null) return;
            _effect.RefractionIndex = e.NewValue;
        }

		private void AddDrop(object sender, MouseEventArgs args)
		{
			Point point = Mouse.GetPosition(_overlayCanvas);
			point.X = (point.X / _overlayCanvas.ActualWidth) * MeshDivisions;
			point.Y = (point.Y / _overlayCanvas.ActualHeight) * MeshDivisions;
			_effect.PutTurbulence(point);
		}

    }
}